package com.house365.xxx.newhouse.namecheck.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.house365.xxx.newhouse.namecheck.bean.PrjBean;
import com.house365.xxx.newhouse.namecheck.bean.ResultBean;
import com.house365.xxx.newhouse.namecheck.bean.WordBean;
import com.house365.xxx.newhouse.namecheck.util.DistanceCalculate;
import com.house365.xxx.newhouse.namecheck.util.PinyinToolkit;

@Component
public class DistanceChecker {
	private static Log logger = LogFactory.getLog(DistanceChecker.class);
	@Autowired
	private NewProjectBeanLoader projectBeanLoader;

	public void init() {
		projectBeanLoader.load();
	}

	// 中文匹配--》拼音匹配--》首字母匹配--》tag
	private HashSet<WordBean> getWordsWB(int city, String word) {
		HashSet<WordBean> setWB = new HashSet<WordBean>();
		boolean isHanziFirst = isHanziFirst(word);
		if (isHanziFirst) {
			//对地块搜索进行处理
			if (word.startsWith("地") || word.equals("地块")) {
				List<WordBean> resultFromPY = getWordsByPY(city, PinyinToolkit.cn2Spell(word));
				setWB.addAll(resultFromPY);
				if (setWB.isEmpty()) {
					List<WordBean> resultFromName = getWordsByName(city, word);
					setWB.addAll(resultFromName);
				}
			} else {
				List<WordBean> resultFromName = getWordsByName(city, word);
				setWB.addAll(resultFromName);
				if (setWB.isEmpty()) {
					List<WordBean> resultFromPY = getWordsByPY(city, PinyinToolkit.cn2Spell(word));
					setWB.addAll(resultFromPY);
				}
			}
		} else {
			List<WordBean> resultFromPY = getWordsByPY(city, PinyinToolkit.cn2Spell(word));
			setWB.addAll(resultFromPY);
			if (setWB.isEmpty()) {
				List<WordBean> resultFromName = getWordsByName(city, word);
				setWB.addAll(resultFromName);
			}
		}

		if (setWB.isEmpty()) {
			List<WordBean> resultFromPYFst = getWordsByPYFst(city, PinyinToolkit.cn2FirstSpell(word));
			setWB.addAll(resultFromPYFst);
		}
		if (setWB.isEmpty()) {
			List<WordBean> resultFromTag = getWordsByTag(city, word);
			setWB.addAll(resultFromTag);
		}

		return setWB;
	}

	private boolean isHanziFirst(String word) {
		boolean result = false;
		int hanziCount = PinyinToolkit.countHanzi(word);
		if (hanziCount > 0) {
			result = true;
		}
		return result;
	}

	public List<ResultBean> getWords(int city, String word) {
		List<WordBean> lstTempWB = new ArrayList<>();
		lstTempWB.addAll(getWordsWB(city, word));
		// String fWord = filter.filter(word);
		// if (!word.equalsIgnoreCase(fWord)) {
		// lstTempWB.addAll(getWordsWB(city, filter.filter(word)));
		// }
		Collections.sort(lstTempWB);
		int size1 = 100 > lstTempWB.size() ? lstTempWB.size() : 100;

		List<ResultBean> lstRSt = new ArrayList<ResultBean>();
		List<WordBean> lstWB = lstTempWB.subList(0, size1);
		for (WordBean wb : lstWB) {
			lstRSt.add(new ResultBean(wb));
		}
		Collections.sort(lstRSt);
		int size = 30 > lstRSt.size() ? lstRSt.size() : 30;
		List<ResultBean> lstRS = lstRSt.subList(0, size);
		return lstRS;
	}

	public List<WordBean> getWordsByName(int city, String word) {
		HashSet<PrjBean> prjBeans = projectBeanLoader.getSetPrjBean(city);
		if (prjBeans != null) {
			return findWordsByWordDis(word, prjBeans, null, CheckType.HANZI);
		}
		return new ArrayList<WordBean>();
	}

	public List<WordBean> getWordsByPY(int city, String py) {
		List<WordBean> lstRs = new ArrayList<WordBean>();
		HashSet<PrjBean> prjBeans = projectBeanLoader.getSetPrjBean(city);
		if (prjBeans != null) {
			List<String> lstRPY = PinyinToolkit.rMatch(py);
			if (!lstRPY.isEmpty()) {
				lstRs = findWordsByWordDis(py, prjBeans, lstRPY, CheckType.PINYIN);
			}
		}
		return lstRs;
	}

	public List<WordBean> getWordsByPYFst(int city, String pyFst) {
		List<WordBean> lstRs = new ArrayList<WordBean>();
		HashSet<PrjBean> prjBeans = projectBeanLoader.getSetPrjBean(city);
		if (prjBeans != null) {
			lstRs = findWordsByWordDis(pyFst, prjBeans, null, CheckType.PINYINSX);
		}
		return lstRs;
	}

	public List<WordBean> getWordsByTag(int city, String word) {
		List<WordBean> lstRs = new ArrayList<WordBean>();
		HashSet<PrjBean> prjBeans = projectBeanLoader.getSetPrjBean(city);
		if (prjBeans != null) {
			lstRs = findWordsByWordDis(word, prjBeans, null, CheckType.TAG);
		}
		return lstRs;
	}

	private List<WordBean> findWordsByWordDis(String word, HashSet<PrjBean> dic, List<String> lstPY, CheckType type) {
		List<WordBean> lstRs = new ArrayList<WordBean>();
		for (PrjBean prjBean : dic) {
			float matchRate = 0;
			String name = prjBean.getPrj_itemname_search();
			int sale_status = prjBean.getSale_status();
			int middle_sort = prjBean.getMiddle_sort();
			switch (type) {
			case PINYIN:
				name = prjBean.getPrj_pinyin();
				break;
			case PINYINSX:
				name = prjBean.getPyfst();
				break;
			case TAG:
				name = prjBean.getTag();
				break;
			default:
				break;
			}
			String lw = name.toLowerCase();
			if (lw.startsWith(word)) {
				matchRate = 1.0f;
			} else if (lw.contains(word)) {
				matchRate = 0.9f;
			} else {
				float myLen = 0;
				float dicLen = 0;
				float dis = 0;
				if (lstPY != null) {
					List<String> lstWordsInDic = prjBean.getLstPY();
					dicLen = lstWordsInDic.size();
					myLen = lstPY.size();
					dis = DistanceCalculate.getSentenceDistance(lstPY.toArray(new String[(int) myLen]),
							lstWordsInDic.toArray(new String[(int) dicLen]));
				} else {
					dicLen = lw.length();
					myLen = word.length();
					dis = DistanceCalculate.getWordDistance(word, lw);
				}
				float len = myLen > dicLen ? myLen : dicLen;
				float matchSize = len - dis;
				matchRate = matchSize / dicLen;
			}
			if (type == CheckType.TAG) {
				matchRate = matchRate * 0.8f;
			}
			if (matchRate > 0.2) {
				lstRs.add(new WordBean(name, matchRate + getMatchRateAdd(sale_status,middle_sort), prjBean));
			}
		}
		return lstRs;
	}

	/**
	 * @param sale_status 楼盘状态
	 * @param middle_sort 排序字段
	 * @return 权重加分
	 */
	private float getMatchRateAdd(int sale_status,int middle_sort) {
		//楼盘状态加持
		float rate = 0;
		switch (sale_status) {
		case 3:
			rate = 0.4f;
			break;
		case 2:
			rate = 0.2f;
			break;
		default:
			break;
		}
		return rate+middle_sort*99.0f;
	}
}
