package com.house365.xxx.newhouse.namecheck.servlet;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.CDATA;
import org.jdom.Document;
import org.jdom.Element;
import org.springframework.context.ApplicationContext;

import com.house365.xxx.common.config.SystemStarter;
import com.house365.xxx.common.util.DefDocOutputDevice;
import com.house365.xxx.common.util.DocConverter;
import com.house365.xxx.newhouse.namecheck.bean.PrjBean;
import com.house365.xxx.newhouse.namecheck.bean.ResultBean;
import com.house365.xxx.newhouse.namecheck.core.DistanceChecker;

public class ProjectQuery extends HttpServlet {

	private static final long serialVersionUID = -1204947606286376668L;
	private static Log logger = LogFactory.getLog(ProjectQuery.class);
	private static SystemStarter loader = new SystemStarter();
	private static DistanceChecker checker;

	public void init() throws ServletException {
		try {
			loader.start();
			ApplicationContext applicationContext = loader
					.getApplicationContext();
			checker = applicationContext.getBean(DistanceChecker.class);
			checker.init();
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
	}

	public static SystemStarter getLoader() {
		return loader;
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response) {
		doPost(request, response);
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response) {
		try {
			request.setCharacterEncoding("UTF-8");
		} catch (UnsupportedEncodingException e) {
			logger.error(e.getMessage(), e);
		}
		String city = request.getParameter("city");
		String project = request.getParameter("project").trim().toLowerCase();
		if (StringUtils.isNotBlank(city) && StringUtils.isNotBlank(project)) {
			List<ResultBean> lstResult = checker.getWords(
					Integer.valueOf(city), project);
			Document document = new Document();
			Element recordSet = getElement(lstResult);
			document.addContent(recordSet);
			String xml = null;
			try {
				xml = DocConverter.doc2String(document);
				DefDocOutputDevice.showDoc(response, xml, "UTF-8");
			} catch (IOException e) {
				logger.error(e.getMessage(), e);
			}

		}
	}

	private Element getElement(List<ResultBean> lstResultBean) {
		Element root = new Element("RecordSet");
		root.addContent(new Element("MSG").addContent("ok"));
		root.addContent(new Element("TOTAL"));
		for (ResultBean rsb : lstResultBean) {
			PrjBean pBean = rsb.getPrjBean();
			Element record = new Element("Record");
			record.addContent(new Element("prj_id").addContent(new CDATA(String
					.valueOf(pBean.getPrj_id()))));
			record.addContent(new Element("prj_city").addContent(new CDATA(
					String.valueOf(pBean.getPrj_city()))));
			record.addContent(new Element("prj_listid").addContent(new CDATA(
					String.valueOf(pBean.getPrj_listid()))));
			record.addContent(new Element("prj_channel").addContent(new CDATA(
					String.valueOf(pBean.getPrj_channel()))));
			record.addContent(new Element("prj_channel_str")
					.addContent(new CDATA(pBean.getPrj_channel_str())));
			record.addContent(new Element("prj_itemname").addContent(new CDATA(
					pBean.getPrj_itemname())));
            record.addContent(new Element("prj_promotionname").addContent(new CDATA(
                    pBean.getPrj_promotionname())));
			record.addContent(new Element("prj_itemname_search")
					.addContent(new CDATA(pBean.getPrj_itemname_search())));
			record.addContent(new Element("prj_pinyin").addContent(new CDATA(
					pBean.getPrj_pinyin())));
			record.addContent(new Element("pinyin").addContent(new CDATA(
					String.valueOf(pBean.getPinyin()))));
			record.addContent(new Element("prj_dist_str").addContent(new CDATA(
					pBean.getPrj_dist_str())));
			record.addContent(new Element("prj_block_str")
					.addContent(new CDATA(pBean.getPrj_block_str())));
			record.addContent(new Element("prj_loc").addContent(new CDATA(pBean
					.getPrj_loc())));
			record.addContent(new Element("prj_kfsname").addContent(new CDATA(
					pBean.getPrj_kfsname())));
			record.addContent(new Element("tag").addContent(new CDATA(pBean
					.getTag())));
			record.addContent(new Element("dateline").addContent(new CDATA(
					String.valueOf(pBean.getDateline()))));
			record.addContent(new Element("del").addContent(new CDATA(String
					.valueOf(pBean.getDel()))));
			record.addContent(new Element("wid").addContent(new CDATA(String
					.valueOf(pBean.getWid()))));
			record.addContent(new Element("info").addContent(new CDATA(String
					.valueOf(pBean.getInfo()))));
			record.addContent(new Element("info2").addContent(new CDATA(String
					.valueOf(pBean.getInfo2()))));
			record.addContent(new Element("prj_ob").addContent(new CDATA(String
					.valueOf(pBean.getPrj_ob()))));
			record.addContent(new Element("word_score").addContent(new CDATA(
					String.valueOf(rsb.getWordRatio()))));
			record.addContent(new Element("sale_status").addContent(new CDATA(
					String.valueOf(pBean.getSale_status()))));
			record.addContent(new Element("middle_sort").addContent(new CDATA(
					String.valueOf(pBean.getMiddle_sort()))));
			root.addContent(record);
		}
		return root;
	}

}
