package com.house365.xxx.newhouse.namecheck.util;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.commons.lang.StringUtils;
import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;

public class PinyinToolkit {
	public static String lRegEx = "(a[io]?|ou?|e[inr]?|ang?|ng|[bmp](a[io]?|[aei]ng?|ei|ie?|ia[no]|o|u)|pou|me|m[io]u|[fw](a|[ae]ng?|ei|o|u)|fou|wai|[dt](a[io]?|an|e|[aeio]ng|ie?|ia[no]|ou|u[ino]?|uan)|dei|diu|[nl](a[io]?|ei?|[eio]ng|i[eu]?|i?ang?|iao|in|ou|u[eo]?|ve?|uan)|nen|lia|lun|[ghk](a[io]?|[ae]ng?|e|ong|ou|u[aino]?|uai|uang?)|[gh]ei|[jqx](i(ao?|ang?|e|ng?|ong|u)?|u[en]?|uan)|([csz]h?|r)([ae]ng?|ao|e|i|ou|u[ino]?|uan)|[csz](ai?|ong)|[csz]h(ai?|uai|uang)|zei|[sz]hua|([cz]h|r)ong|y(ao?|[ai]ng?|e|i|ong|ou|u[en]?|uan))";
	public static String rRegEx = "(a[io]?|ou?|e[inr]?|ang?|ng|[bmp](a[io]?|[aei]ng?|ei|ie?|ia[no]|o|u)|pou|me|m[io]u|[fw](a|[ae]ng?|ei|o|u)|fou|wai|[dt](a[io]?|an|e|[aeio]ng|ie?|ia[no]|ou|u[ino]?|uan)|dei|diu|[nl](a[io]?|ei?|[eio]ng|i[eu]?|i?ang?|iao|in|ou|u[eo]?|ve?|uan)|nen|lia|lun|[ghk](a[io]?|[ae]ng?|e|ong|ou|u[aino]?|uai|uang?)|[gh]ei|[jqx](i(ao?|ang?|e|ng?|ong|u)?|u[en]?|uan)|([csz]h?|r)([ae]ng?|ao|e|i|ou|u[ino]?|uan)|[csz](ai?|ong)|[csz]h(ai?|uai|uang)|zei|[sz]hua|([cz]h|r)ong|y(ao?|[ai]ng?|e|i|ong|ou|u[en]?|uan))$";

	public static List<String> lMatch(String py) {
		int tag = 0;
		LinkedList<String> lstPY = new LinkedList<String>();
		for (int i = py.length(); i > 0; i = i - tag) {
			Pattern pat = Pattern.compile(lRegEx);
			Matcher matcher = pat.matcher(py);
			if (matcher.find()) {
				lstPY.add(matcher.group());
				tag = matcher.end() - matcher.start();
				py = py.substring(tag);
			}else{
				break;
			}
		}
		return lstPY;
	}

	public static List<String> rMatch(String py) {
		int tag = 0;
		LinkedList<String> lstPY = new LinkedList<String>();
		for (int i = py.length(); i > 0; i = tag) {
			Pattern pat = Pattern.compile(rRegEx);
			Matcher matcher = pat.matcher(py);
			if (matcher.find()) {
				lstPY.push(matcher.group());
				tag = matcher.start();
				py = py.substring(0, matcher.start());
			}else{
				break;
			}
		}
		return lstPY;
	}

	// 判断查询词中是否都为字母
	public static boolean judgeLetter(String query) {
		query = query.toLowerCase();
		char[] array = query.toCharArray();
		for (int i = 0; i < array.length; i++) {
			if (array[i] < 'a' || array[i] > 'z') {
				return false;
			}
		}
		return true;
	}

	// 判断查询是否都为汉字
	public static boolean judgeHanzi(String query) {
		for (int i = 0; i < query.length(); i++) {
			if (!query.substring(i, i + 1).matches("[\\u4e00-\\u9fa5]+")) {
				return false;
			}
		}
		return true;
	}

    // 计算汉字个数
    public static int countHanzi(String word) {
        int count = 0;
        for (int i = 0; i < word.length(); i++) {
            if (word.substring(i, i + 1).matches("[\\u4e00-\\u9fa5]+")) {
                count++;
            }
        }
        return count;
    }

	/**
	 * 获取汉字串拼音首字母，英文字符不变
	 * 
	 * @param chinese
	 *            汉字串
	 * @return 汉语拼音首字母
	 */
	public static String cn2FirstSpell(String chinese) {
		StringBuffer pybf = new StringBuffer();
		char[] arr = chinese.toCharArray();
		HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
		defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
		defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
		for (int i = 0; i < arr.length; i++) {
			if (arr[i] > 128) {
				try {
					String[] _t = PinyinHelper.toHanyuPinyinStringArray(arr[i],
							defaultFormat);
					if (_t != null) {
						pybf.append(_t[0].charAt(0));
					}
				} catch (BadHanyuPinyinOutputFormatCombination e) {
					e.printStackTrace();
				}
			} else {
				pybf.append(arr[i]);
			}
		}
		return pybf.toString().replaceAll("\\W", "").trim();
	}

	/**
	 * 获取汉字串拼音，英文字符不变
	 * 
	 * @param chinese
	 *            汉字串
	 * @return 汉语拼音
	 */
	public static String cn2Spell(String chinese) {
		StringBuffer pybf = new StringBuffer();
		char[] arr = chinese.toCharArray();
		HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
		defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
		defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
		for (int i = 0; i < arr.length; i++) {
			if (arr[i] > 128) {
				try {
					String[] _t = PinyinHelper.toHanyuPinyinStringArray(arr[i],
							defaultFormat);
					if (_t != null) {
						pybf.append(_t[0]);
					}
				} catch (BadHanyuPinyinOutputFormatCombination e) {
					e.printStackTrace();
				}
			} else {
				pybf.append(arr[i]);
			}
		}
		return pybf.toString();
	}

	/**
	 * 获取汉字串拼音，英文字符不变
	 * 
	 * @param phrases
	 *            汉字串
	 * @return 汉语拼音
	 */
	public static List<String> getLstCn2Spell(String phrases) {
		List<String> lstPY = new ArrayList<String>();
        String[] arrPhrase = StringUtils.split(phrases, ",");
        int charCount = 0;
        for (String phrase : arrPhrase) {
            charCount += phrase.length();
        }
        char[] arr = new char[charCount];
        int arrIndex = 0;
        for (String phrase : arrPhrase) {
            for (int index = 0; index < phrase.length(); index++) {
                arr[arrIndex++] = phrase.charAt(index);
            }
        }
		HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
		defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
		defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
		for (int i = 0; i < arr.length; i++) {
			if (arr[i] > 128) {
				try {
					String[] _t = PinyinHelper.toHanyuPinyinStringArray(arr[i],
							defaultFormat);
					if (_t != null) {
						lstPY.add(_t[0]);
					}
				} catch (BadHanyuPinyinOutputFormatCombination e) {
					e.printStackTrace();
				}
			} else {
				lstPY.add(String.valueOf(arr[i]));
			}
		}
		return lstPY;
	}

}
